import { queryOne, create, update } from '../../services/course';
import { message } from 'antd';
import pathToRegexp from 'path-to-regexp';
import { Codes } from '../../utils/config';

export default {
  namespace: 'courseDetail',

  state: {
    filters: {},
    operType: 'create',
    currentItem: {},
    lessonModalVisible: false,
    resourceModalVisible: false,
    supportModalVisible: false,
    itemLoading: false,
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen(({ pathname, state, ...rest }) => {
        const match = pathToRegexp('/product/course/edit/:id').exec(pathname);
        if (match) {
          dispatch({ type: 'query', payload: { id: match[1] } });
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'update' } });
        }
        if (pathname === '/product/course/add') {
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'create' } });
        }
      });
    }
  },

  effects: {
    * query ({ payload }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { itemLoading: true } });
      const { data, success } = yield call(queryOne, payload);
      if (success) {
        yield put({ type: 'querySuccess', payload: { ...data } });
      }
      yield put({ type: 'changeLoading', payload: { itemLoading: false } });
    },
    * create ({ payload, callback }, { call, put }) {
      // 创建课程，默认状态为NORMAL
      const { data, success } = yield call(create, { ...payload, status: Codes.CODE_NORMAL });
      if (success) {
        message.success('创建成功!');
        yield put({ type: 'clearPage' });
        if (callback) {
          callback();
        }
      } else {
        message.error('创建失败!');
      }
    },
    * update ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(update, payload);
      if (success) {
        message.success('更新成功!');
        yield put({ type: 'clearPage' });
        if (callback) {
          callback();
        }
      } else {
        message.error('更新失败!');
      }
    }
  },

  reducers: {
    changeLoading(state, { payload }) {
      return { ...state, ...payload };
    },

    querySuccess(state, { payload }) {
      return { ...state, currentItem: payload };
    },

    saveFilters(state, { payload: filters }) {
      return { ...state, filters };
    },

    showLessonModal(state, { payload }) {
      return { ...state, ...payload, lessonModalVisible: true };
    },

    hideLessonModal(state) {
      return { ...state, lessonModalVisible: false };
    },

    showSupportModal(state, { payload }) {
      return { ...state, ...payload, supportModalVisible: true };
    },

    hideSupportModal(state) {
      return { ...state, supportModalVisible: false };
    },

    showResourceModal(state, { payload }) {
      return { ...state, ...payload, resourceModalVisible: true };
    },

    hideResourceModal(state) {
      return { ...state, resourceModalVisible: false };
    },

    saveOperType(state, { payload }) {
      return { ...state, ...payload };
    },

    saveCoverImg(state, { payload: { cvImg } }) {
      const currentItem = { ...state.currentItem, cvImg };
      return { ...state, resourceModalVisible: false, currentItem };
    },

    saveBackgroundImg(state, { payload: { bgImg } }) {
      const currentItem = { ...state.currentItem, bgImg };
      return { ...state, resourceModalVisible: false, currentItem };
    },

    saveLessonList(state, { payload: { lessonList } }) {
      const currentItem = { ...state.currentItem, lessonList };
      return { ...state, lessonModalVisible: false, currentItem };
    },

    saveSupportList(state, { payload: { supportList } }) {
      const currentItem = { ...state.currentItem, supportList };
      return { ...state, supportModalVisible: false, currentItem };
    },

    clearPage(state) {
      return { ...state, currentItem: {}, itemLoading: false };
    }
  }
}
